class Library(object):
	def __init__(self, layout):
		self.layout = layout

def Slot(name, wrapped=False):
	def wrapper(fn):
		def slot(*args, **kwargs):
			return fn(*args, **kwargs)

		slot.slot = name
		slot.wrapped = wrapped
		return slot

	return wrapper


from library import Library
from ..nodes import Generator, Node
import operator

class Scrollable(Library):
	class Scrolled(Node):
		container = True
		text_container = False
		properties = ["x", "y"]

		def __init__(self, x=0, y=0, children=[]):
			super(Scrollable.Scrolled, self).__init__(children=children)
			self.x = x
			self.y = y

		def render(self, dry_run=False):
			if len(self.children) != 1:
				raise ValueError("<Scrolled> can contain only one node")

			x = self.layout.calcRelativeSize(self.x, self.render_parent_width , self.render_stretch)
			y = self.layout.calcRelativeSize(self.y, self.render_parent_height, self.render_stretch)

			offset = list(self.render_offset)
			offset = map(operator.sub, offset, (x, y))
			offset = tuple(offset)

			return self.renderChild(
				self.children[0], dry_run=dry_run,

				offset=offset,
				plus_size=(x, y),
				boundary_left_top=self.render_boundary_left_top,
				boundary_right_bottom=self.render_boundary_right_bottom,
				parent_width=self.render_parent_width,
				parent_height=self.render_parent_height,
				stretch=self.render_stretch
			)


from screen import *
from layout import *
from layout.libs import register

def easyRender(xml):
	from layout.xml_parser import gatherLibs, fromXml
	from layout.libs import getAdditionalNodes, getAdditionalSlots, render

	libs = gatherLibs(xml)
	nodes = getAdditionalNodes(libs)
	slots = getAdditionalSlots(libs)
	root = fromXml(xml, additional_nodes=nodes, global_slots=slots)

	layout = Layout(root)
	render(layout, libs)
	return layout


from library import Library
from ..nodes import Generator

class Conditional(Library):
	def __init__(self, layout):
		super(Conditional, self).__init__(layout)


	class If(Generator):
		container = True
		text_container = False
		properties = ["is", "with", "then", "else"]

		def init(self, is_, with_=None, then=None, else_=None):
			self.is_ = is_
			self.with_ = with_
			self.then = then
			self.else_ = else_

			if self.with_ is None:
				if then is not None:
					raise ValueError("Unexpected 'then' argument of <If>")
				elif else_ is not None:
					raise ValueError("Unexpected 'else' argument of <If>")
			else:
				if then is None:
					raise ValueError("Expected 'then' argument of <If with>")
				elif else_ is None:
					raise ValueError("Expected 'else' argument of <If with>")

		def onGenerate(self):
			from ..xml_parser import handleElement

			result = self.evaluate()

			if self.with_ is None:
				# <If is="condition">
				#   <Then>
				#     <A />
				#   </Then>
				#   <Else>
				#     <B />
				#   </Else>
				# </If>

				res = []
				for child in self.children:
					if child.tag == "Then":
						if result:
							res += handleElement(child, self.defines, self.slots, self.additional_nodes, global_slots=self.global_slots)
					elif child.tag == "Else":
						if not result:
							res += handleElement(child, self.defines, self.slots, self.additional_nodes, global_slots=self.global_slots)
					else:
						raise ValueError("Unexpected %s inside <If>" % child)
			else:
				# <If with="slot" is="condition" then="A" else="B">
				#   <Slot name="slot" />
				# </If>

				slots = dict(**self.slots)
				if result:
					slots[self.with_] = self.then
				else:
					slots[self.with_] = self.else_

				res = []
				for child in self.children:
					if child.tag == ("Then", "Else"):
						raise ValueError("Unexpected %s inside <If with>" % child)

					res += handleElement(child, self.defines, slots, self.additional_nodes, global_slots=self.global_slots)

			return res

		def evaluate(self):
			return bool(self.is_) and self.is_ != "False"


	class _Inside(Generator):
		container = True
		text_container = False
		properties = []

		def onGenerate(self):
			from ..xml_parser import handleElement

			res = []
			for child in self.children:
				res += handleElement(child, self.defines, self.slots, self.additional_nodes, global_slots=self.global_slots)

			return res

	class Then(_Inside):
		pass
	class Else(_Inside):
		pass


from rect import Rect

class AlignRight(Rect):
	container = True
	text_container = False
	properties = ["height"]

	def render(self, dry_run=False):
		if len(self.children) != 1:
			raise ValueError("<AlignRight> can contain only one node")

		# Get child size
		child_width, child_height = self.guessContainerSize()

		# Guess container size
		if self.height is None:
			height = child_height
		else:
			height = self.height

		width, height = super(AlignRight, self).render(dry_run=dry_run, height=height)
		self.renderChildren(width, height, dry_run=dry_run, child_width=child_width)

		return width, height

	def guessContainerSize(self):
		width, height = super(AlignRight, self).render(dry_run=True, width=self.width, height=self.height or 0)
		return self.renderChildren(width, height, dry_run=True)

	def renderChildren(self, width, height, dry_run=False, child_width=None):
		x1, y1 = self.render_offset
		x2, y2 = x1 + width, y1 + height

		if child_width is not None:
			x1 = x2 - child_width

		child = self.children[0]
		return self.renderChild(
			child, dry_run=dry_run,

			offset=(x1, y1),

			boundary_left_top=[
				x1,
				y1 if self.height is not None else self.render_boundary_left_top[1]
			],
			boundary_right_bottom=[
				x2,
				y2 if self.height is not None else self.render_boundary_right_bottom[1]
			],

			parent_width=width,
			parent_height=height,

			stretch=self.render_stretch
		)


from library import Library
import all

def getLibInfo(lib):
	if isinstance(lib, type) and issubclass(lib, Library):
		return lib

	if not hasattr(all, lib):
		raise AttributeError("No %s lib" % lib)
	return getattr(all, lib)

def getDependencies(libs):
	res = []
	for lib in libs:
		dependencies = getattr(getLibInfo(lib), "dependencies", [])
		res += getDependencies(dependencies)
		res.append(lib)
	return res

def getAdditionalNodes(libs):
	from ..nodes import Node, Generator

	nodes = {}
	for lib in getDependencies(libs):
		info = getLibInfo(lib)

		for obj in dir(info):
			try:
				if issubclass(getattr(info, obj), (Node, Generator)):
					nodes[obj] = getattr(info, obj)
			except TypeError:
				pass

	return nodes

def getAdditionalSlots(libs):
	slots = {}
	for lib in getDependencies(libs):
		info = getLibInfo(lib)

		for obj in dir(info):
			try:
				if hasattr(getattr(info, obj), "slot"):
					slots[getattr(info, obj).slot] = getattr(info, obj).__func__
			except TypeError, e:
				pass

	return slots

def register(name, lib):
	setattr(all, name, lib)

def render(layout, libs):
	libs = getDependencies(libs)

	lib_instances = {}
	for lib in libs:
		lib_instances[lib] = getLibInfo(lib)(layout)

	layout.libs = lib_instances
	layout.render(force=True)

	if any(hasattr(lib, "loop") for lib in lib_instances.values()):
		while True:
			for lib in lib_instances.values():
				if hasattr(lib, "beforeLoop"):
					lib.beforeLoop(lib_instances)
			for lib in lib_instances.values():
				if hasattr(lib, "loop"):
					lib.loop(lib_instances)

			layout.render(clear=False)


class Generator(object):
	container = False
	text_container = False

	def __init__(self, slots, children=[], **kwargs):
		self.children = children
		self.slots = slots

		self.parent = None
		self.gen_parent = None

		self._cached = None
		self._kwargs = kwargs
		self._initted = False

	def init(self):
		pass

	def generate(self):
		if not self._initted:
			self.init(**self._kwargs)
			self._initted = True

		if self._cached is None:
			self._cached = self.onGenerate()

		return self._cached

	def onGenerate(self):
		raise NotImplementedError


	def revoke(self):
		self._cached = None

		node = self.parent
		while node is not None:
			node._revoked = True
			node = node.parent


from node import Node

class Text(Node):
	container = False
	text_container = True
	properties = ["width", "bg", "color", "bright", "fill"]

	def __init__(self, value, width=None, bg=None, color=None, bright=False, fill=False, **kwargs):
		super(Text, self).__init__(value=value, **kwargs)

		self.width = width
		self.bg = bg
		self.color = color
		self.bright = bright is not False
		self.fill = fill is not False

	def render(self, dry_run=False):
		if self.width is not None:
			width = self.width
			width = self.layout.calcRelativeSize(width, self.render_parent_width, self.render_stretch)
		else:
			width = len(self.value)

		width += self.render_plus_size[0]

		x, y = self.render_offset

		if not (self.render_boundary_left_top[0] <= x <= x + width <= self.render_boundary_right_bottom[0]):
			return width, 1
		elif not (self.render_boundary_left_top[1] <= y < self.render_boundary_right_bottom[1]):
			return width, 1

		bg = self.inherit("bg")
		color = self.inherit("color")
		bright = self.inherit("bright")

		if color is not None:
			if not dry_run:
				if self.fill:
					self.layout.screen.fill(x, y, x + width, y + 1, char=self.value, style=lambda s: self.layout.screen.colorize(s, bg=bg, fg=color, bright=bright))
				else:
					self.layout.screen.printAt(self.layout.screen.colorize(self.value, bg=bg, fg=color, bright=self.bright), x, y)
					self.layout.screen.printAt(self.layout.screen.colorize(" " * int(width - len(self.value)), bg=bg), x + len(self.value), y)

		return width, 1


class PatchProxy(object):
	def __init__(self, instance):
		object.__setattr__(self, "cls", instance._patched_obj)
		object.__setattr__(self, "instance", instance)

	def __getattribute__(self, name):
		cls = object.__getattribute__(self, "cls")
		instance = object.__getattribute__(self, "instance")

		if name in cls._unpatched:
			value = cls._unpatched[name]
		else:
			value = getattr(cls, name)

		if callable(value) and value.__self__ is None:
			# Unbound
			value = value.__get__(instance, cls)

		return value

	def __setattr__(self, name, value):
		cls = object.__getattribute__(self, "cls")
		instance = object.__getattribute__(self, "instance")

		if name in instance._unpatched:
			instance._unpatched[name] = value
		else:
			setattr(cls, name, value)


def patch(*objects):
	def wrapper(cls):
		if not isinstance(cls, type):
			raise TypeError("Can only @patch a class")

		names = set(dir(cls)) # All variables
		names -= set(dir(type)) # Remove some __vars__
		if "__weakref__" in names:
			names.remove("__weakref__") # Remove __weakref__

		# Bring back some methods defined by type()
		names.add("__init__")

		for obj in objects:
			if not hasattr(obj, "_unpatched"):
				obj._unpatched = {}
				obj._patched_obj = obj

			for name in names:
				if name not in cls.__dict__:
					continue

				if name not in obj._unpatched and hasattr(obj, name):
					obj._unpatched[name] = getattr(obj, name)

				setattr(obj, name, cls.__dict__[name])

		return obj

	return wrapper


patch.super = PatchProxy


from lxml import etree

import nodes

NoDefault = nodes.NoDefault

def gatherLibs(node):
	if isinstance(node, (str, unicode)):
		parser = etree.XMLParser(recover=True)
		node = etree.fromstring(node, parser=parser)

	libs = []
	for node in node.findall("Use"):
		if "lib" in node.attrib:
			libs.append(node.attrib["lib"])
	return libs

def fromXml(code, additional_nodes={}, global_slots={}):
	global_slots["__unset__"] = None

	parser = etree.XMLParser(recover=True)
	root = etree.fromstring(code, parser=parser)

	# Handle <Define>
	define_nodes = root.findall("Define")

	for node in define_nodes:
		if "name" not in node.attrib:
			raise ValueError("<Define> without 'name' attribute")
		elif getattr(nodes, node.tag, None) is not None:
			raise ValueError("Cannot <Define> core node %s" % node.tag)
		elif additional_nodes.get(node.tag, None) is not None:
			raise ValueError("Cannot <Define> core node %s" % node.tag)

	define_nodes.sort(key=lambda define: define.attrib["name"])
	for i in range(len(define_nodes) - 1):
		a = define_nodes[i + 0].attrib["name"]
		b = define_nodes[i + 1].attrib["name"]
		if a == b:
			raise ValueError("Non-unique <Define name='%s'>" % a)

	defines = {}
	for node in define_nodes:
		slots = {}
		container = ""

		children = list(node)
		for child in children:
			if child.tag == "Slot" and "define" in child.attrib:
				name = child.attrib["define"]
				if name in global_slots:
					raise ValueError("Defining special slot :%s" % name)

				slots[name] = child.attrib.get("default", NoDefault)
				if slots[name] is NoDefault:
					slot = child.attrib.get(":default", NoDefault)
					if slot is not NoDefault:
						if slot != "__unset__":
							raise ValueError("Deprecated: No slot except :__unset__ can be used for :default")
						slots[name] = evaluate(slot, slots={}, global_slots=global_slots)
					else:
						slot = child.attrib.get("::default", NoDefault)
						if slot is not NoDefault:
							slots[name] = evaluatable("default", slot, global_slots=global_slots)

				if name == "":
					if "container" not in child.attrib:
						raise ValueError("Deprecated: Set container=node or container=text for <Define name='' />")
					elif child.attrib["container"].lower() == "node":
						container = "node"
					elif child.attrib["container"].lower() == "text":
						container = "text"
					else:
						raise ValueError("Only 'node' or 'text' values are accepted for 'container' attribute of <Define name='' />")

		children = filter(lambda child: (child.tag != "Slot" or "define" not in child.attrib) and child.tag != etree.Comment, children)

		if len(children) != 1:
			raise ValueError("<Define> must have exactly one child")
		if children[0].tag == "Range":
			raise ValueError("<Define> must have exactly one child: cannot guarantee that <Range> is always one child")

		defines[node.attrib["name"]] = nodes.fromXml(elem=children[0], slots=slots, defines=defines, name=node.attrib["name"], container=container, additional_nodes=additional_nodes, global_slots=global_slots)

	# Handle <Global>
	global_nodes = root.findall("Global")
	additional_global_slots = {}
	for node in global_nodes:
		for name, value in node.attrib.items():
			# Set name=value
			if name.startswith("::"):
				name = name[2:]
				value = evaluatable(name, value, global_slots=global_slots)
			elif name.startswith(":"):
				name = name[1:]
				value = evaluate(value, slots={}, global_slots=global_slots)

			additional_global_slots[name] = value

	global_slots.update(additional_global_slots)

	return handleElement(root, defines, slots={}, additional_nodes=additional_nodes, global_slots=global_slots)[0]

def handleElement(node, defines, slots, additional_nodes, global_slots):
	if node.tag in ["Define", "Use", "Global"]:
		return []
	elif node.tag is etree.Comment:
		return []
	elif node.tag == "Slot":
		if "define" in node.attrib:
			return []

		name = node.attrib.get("name", "")
		value = evaluate(name, slots=slots, global_slots=global_slots)
		if not isinstance(value, (nodes.Node, nodes.Generator)):
			raise ValueError("Unexpected %s slot :%s" % (type(value), name))
		elif value is NoDefault:
			raise ValueError("Required slot :%s was not passed (from <%s>)" % (name, node.tag))

		return [value]

	attrs = {}
	inheritable = {}
	for attr, value in node.attrib.items():
		if attr.startswith("::"):
			attr = attr[2:]
			value = evaluatable(attr, value, global_slots=global_slots)
		elif attr.startswith(":"):
			attr = attr[1:]
			value = evaluate(value, slots=slots, global_slots=global_slots)

		if attr.startswith("inherit-"):
			attr = attr[len("inherit-"):]
			inheritable[attr] = value
		else:
			# Escape keywods
			if attr in ("from", "is", "with", "else", "in"):
				attr += "_"

			attrs[attr] = value

	ctor = getattr(nodes, node.tag, None)
	if ctor is None:
		ctor = defines.get(node.tag, None)
	if ctor is None:
		ctor = additional_nodes.get(node.tag, None)
	if ctor is None:
		raise ValueError("Unknown node <%s>" % node.tag)

	if issubclass(ctor, nodes.Generator):
		return handleGenerator(node, defines, slots, ctor=ctor, attrs=attrs, inheritable=inheritable, additional_nodes=additional_nodes, global_slots=global_slots)

	result = None
	if ctor.text_container:
		text_inside = getTextInside(node, slots=slots, global_slots=global_slots)

		result = ctor(value=text_inside, **attrs)
	elif ctor.container:
		text_inside = getTextInside(node, slots=slots, global_slots=global_slots, allow_nodes=True)
		if text_inside.strip() != "":
			raise ValueError("Text inside <%s>" % node.tag)

		children = []
		for child in node:
			children += handleElement(child, defines, slots, additional_nodes=additional_nodes, global_slots=global_slots)

		result = ctor(children=children, **attrs)
	else:
		if len(node) > 0:
			raise ValueError("Nodes inside <%s>" % node.tag)
		elif node.text is not None and node.text.strip() != "":
			raise ValueError("Text inside <%s>" % node.tag)

		result = ctor(**attrs)

	result.slots = slots
	result.defines = defines
	result.additional_nodes = additional_nodes
	result.inheritable = inheritable
	return [result]

def handleGenerator(node, defines, slots, ctor, attrs, inheritable, additional_nodes, global_slots):
	if ctor.text_container:
		node = ctor(value=getTextInside(node, slots, global_slots=global_slots, allow_nodes=False), slots=slots, **attrs)
	elif ctor.container:
		node = ctor(children=list(node), slots=slots, **attrs)
	else:
		node = ctor(slots=slots, **attrs)

	node.defines = defines
	node.additional_nodes = additional_nodes
	node.global_slots = global_slots
	node.inheritable = inheritable
	return [node]

def getTextInside(node, slots, global_slots, allow_nodes=False):
	text = ""
	had_nodes = False

	for item in node.xpath("child::node()"):
		if isinstance(item, (str, unicode)):
			text += item

			if allow_nodes is None and text.strip() != "" and had_nodes:
				raise ValueError("Nodes and text inside <%s>" % node.tag)
		elif item.tag is etree.Comment:
			pass
		elif item.tag == "Slot":
			name = item.attrib.get("name", "")
			value = evaluate(name, slots=slots, global_slots=global_slots)
			if not isinstance(value, (nodes.Node, nodes.Generator)):
				text += str(value)
			elif allow_nodes is None and had_nodes:
				raise ValueError("Nodes and text inside <%s>" % node.tag)
			elif allow_nodes is False:
				raise ValueError("Nodes inside <%s>" % node.tag)
		else:
			if allow_nodes is None and text.strip() != "":
				raise ValueError("Nodes and text inside <%s>" % node.tag)
			elif allow_nodes is False:
				raise ValueError("Nodes inside <%s>" % node.tag)

			had_nodes = True

	return text


def evaluatable(name, expr, global_slots):
	def func(*args, **kwargs):
		for pos, value in enumerate(args):
			kwargs["arg" + str(pos)] = value

		try:
			return evaluate(expr, slots=kwargs, global_slots=global_slots)
		except Exception, e:
			raise ValueError("Could not evaluate ::%s: %s" % (name, e))

	return func

def evaluate(expr, slots, global_slots):
	if expr == "":
		return slots[""]

	all_slots = {}
	all_slots.update(slots)
	all_slots.update(global_slots)

	from safe_eval import safeEval
	return safeEval(expr, all_slots)


from nodes import Node, Generator

class NodeAPI(object):
	def getBoundaryBox(self):
		node = storage[self]
		if isinstance(node, Node):
			return node.getBoundaryBox()
		else:
			raise self._unexpectedType("getBoundaryBox()")


	@property
	def childNodes(self):
		node = storage[self]
		if isinstance(node, Node):
			return map(createAPI, node._children)
		elif isinstance(node, Generator):
			return map(createAPI, node.generate())
		else:
			raise self._unexpectedType("childNodes")

	@property
	def children(self):
		node = storage[self]
		if isinstance(node, Node):
			return map(createAPI, node.children)
		elif isinstance(node, Generator):
			return map(createAPI, self._get_children_recursively(node))
		else:
			raise self._unexpectedType("children")

	def _get_children_recursively(self, generator):
		res = []
		for node in generator.generate():
			if isinstance(node, Generator):
				res += self._get_children_recursively(node)
			else:
				res.append(node)
		return res


	@property
	def parent(self):
		node = storage[self]
		return createAPI(node.gen_parent)


	def isNode(self):
		node = storage[self]
		return isinstance(node, Node)
	def isGenerator(self):
		node = storage[self]
		return isinstance(node, Generator)


	@property
	def nodeName(self):
		node = storage[self]
		return node.__class__.__name__


	def __getitem__(self, name):
		node = storage[self]
		if name not in node.properties:
			raise ValueError("[]: %s is not a property" % name)
		return getattr(node, name)

	def __setitem__(self, name, value):
		node = storage[self]
		if name not in node.properties:
			raise ValueError("[]: %s is not a property" % name)
		setattr(node, name, value)

		node._completely_revoked = True
		while node is not None:
			node._revoked = True
			node = node.parent


	def _unexpectedType(self, callee):
		node = storage[self]
		return ValueError("%s: Not defined on %s" % (callee, type(node)))


# storage[self] is a way to simulate private scope, so that Node or Generator
# couldn't be accessed from outside
storage = dict()

def createAPI(node):
	class LocalNodeAPI(NodeAPI):
		def __init__(self):
			storage[self] = node
			super(LocalNodeAPI, self).__init__()

	return LocalNodeAPI()


from library import Library
from ..patch import patch
from ..nodes import Node, Generator
from ..api import createAPI

class Events(Library):
	@patch(Node, Generator)
	class Node(object):
		def __init__(self, **kwargs):
			self._events = {}

			for name, handler in kwargs.items():
				if name.startswith("on") and name != "on":
					event = name[2].lower() + name[3:]
					if not callable(handler):
						raise ValueError("Non-callable %s handler was passed to %s: %s" % (name, self, handler))

					del kwargs[name]

					handler = Events.safe(handler, self)

					self.on(event, handler)

			patch.super(self).__init__(**kwargs)

		def on(self, event, handler):
			if event not in self._events:
				self._events[event] = []

			self._events[event].append(handler)

		def emit(self, event, *args, **kwargs):
			for handler in self._events.get(event, []):
				handler(*args, **kwargs)

	@staticmethod
	def safe(handler, node):
		def wrapper(*args, **kwargs):
			return handler(self=createAPI(node), *args, **kwargs)
		return wrapper


from generator import Generator

class NoDefault(object):
	pass

def fromXml(elem, slots, name, defines, container, additional_nodes, global_slots):
	class FromXml(Generator):
		properties = []

		def __new__(cls, **attrs):
			from ..xml_parser import handleElement

			if cls.text_container:
				attrs[""] = attrs["value"]
				del attrs["value"]
			elif cls.container:
				if len(attrs["children"]) > 1:
					raise ValueError("Too many nodes in <%s>" % name)
				elif len(attrs["children"]) == 1:
					rendered = handleElement(attrs["children"][0], defines=defines, slots=attrs["slots"], additional_nodes=additional_nodes, global_slots=global_slots)
					if len(rendered) != 1:
						raise ValueError("Rendered <Slot /> must contain exactly one node")

					attrs[""] = rendered[0]
					del attrs["children"]
				else:
					raise ValueError("Too less nodes in <%s>" % name)

			del attrs["slots"]

			cur_slots = dict(**slots)
			for attr, value in attrs.items():
				if attr not in cur_slots and attr != "":
					raise ValueError("Unexpected slot :%s" % attr)

				cur_slots[attr] = value

			for slot in cur_slots.values():
				if isinstance(slot, NoDefault):
					raise ValueError("Required slot :%s not passed" % slot)

			items = handleElement(elem, defines=defines, slots=cur_slots, additional_nodes=additional_nodes, global_slots=global_slots)
			if len(items) != 1:
				raise ValueError("<Define> must contain exactly one node")
			return items[0]

	FromXml.text_container = container == "text"
	FromXml.container = container == "node"

	return FromXml


class Node(object):
	container = False
	text_container = False

	def __init__(self, children=None, value=[], **kwargs):
		self.render_offset = (None, None)
		self.render_boundary_left_top = [None, None]
		self.render_boundary_right_bottom = [None, None]
		self.parent = None
		self.gen_parent = None
		self.render_stretch = None

		self.inheritable = {}

		self._children = children
		self.value = value
		self._revoked = False
		self._completely_revoked = False
		self.cache_sizes = (None, None)
		self.cache_offset = (None, None)

	@property
	def children(self):
		return self._get_children(self._children, self)

	@children.setter
	def children(self, children):
		self._children = children

	def _get_children(self, old, gen_parent):
		from generator import Generator

		children = []
		for child in old:
			child.parent = self
			child.gen_parent = gen_parent

			if isinstance(child, Generator):
				child.layout = self.layout
				generated = child.generate()

				for subchild in self._get_children(generated, child):
					if not hasattr(subchild, "generated_by"):
						subchild.generated_by = []
					subchild.generated_by.append(child)

					children.append(subchild)
			else:
				children.append(child)

		return children

	def render(self, dry_run=False):
		raise NotImplementedError

	def inherit(self, attr):
		node = self

		value = getattr(node, attr, None)
		try:
			while value.strip().lower() == "inherit":
				parent = node.parent
				if parent is None:
					return None

				value = getattr(parent, attr, None)
				if value is None:
					value = parent.inheritable.get(attr, None)

				# Bypass <Container>
				from container import Container
				if value is None and isinstance(parent, Container):
					value = "inherit"

				node = parent
		except AttributeError:
			pass

		try:
			if isinstance(value, Node) and value.strip().lower() == "inherit":
				return value.inherit(attr)
		except AttributeError:
			pass

		return value


	def renderChild(self, child, dry_run, offset, boundary_left_top, boundary_right_bottom, parent_width, parent_height, stretch, completely_revoked=True, plus_size=(0, 0)):
		child.cache_offset = child.render_offset

		child.render_offset = offset
		child.render_plus_size = plus_size
		child.render_boundary_left_top = boundary_left_top
		child.render_boundary_right_bottom = boundary_right_bottom
		child.render_stretch = stretch
		child.render_parent_width = parent_width
		child.render_parent_height = parent_height
		child.layout = self.layout
		child._completely_revoked = completely_revoked

		sizes = child.render(dry_run=dry_run)
		child.cache_sizes = sizes
		child.cache_offset = offset
		return sizes


	def getBoundaryBox(self):
		return dict(
			left=self.cache_offset[0],
			top=self.cache_offset[1],

			right=self.cache_offset[0] + self.cache_sizes[0],
			bottom=self.cache_offset[1] + self.cache_sizes[1],

			width=self.cache_sizes[0],
			height=self.cache_sizes[1]
		)